#include <algorithm> // 'std::sort'...
#include <array>
#include <atomic> 
#include <iostream>
#include <thread> 

int main(int argc, char** argv) {
	std::array<int, 5> my_array{ 1, 4, 3, 2, 5 };
	std::atomic< std::array<int, 5>* > my_atomic{ &my_array };

	std::thread ascending(
		[&my_atomic]() {
			// Resource is unavailable:
			std::array<int, 5>* my_array = nullptr;
			// Race condition, try to be the first to get the resource:
			do { my_array = my_atomic.exchange(nullptr); } while (my_array == nullptr);
			std::sort(my_array->begin(), my_array->end(),
				[](const int a, const int b) {
					// std::cout << "\tascending a: " << a << " b: " << b << std::endl;
					return a > b;
				});
			// Release resource:
			my_atomic.exchange(my_array);
		});

	std::thread descending(
		[&my_atomic]() {
			// Resource is unavailable:
			std::array<int, 5>* my_array = nullptr;
			// Race condition, try to be the first to get the resource:
			do { my_array = my_atomic.exchange(nullptr); } while (my_array == nullptr);
			std::sort(my_array->begin(), my_array->end(),
				[](const int a, const int b) {
					// std::cout << "\tdescending a: " << a << " b: " << b << std::endl;
					return a < b;
				});
			// Release resource:
			my_atomic.exchange(my_array);
		});

	descending.join();
	ascending.join();

	for (const int i : my_array) // Either ascending or descending based on the "loser thread"
		std::cout << "\t" << i;

	return 0;
}

